﻿using Benner.BeTech.Entities.BeTech;
using Benner.BeTech.Entities.Interfaces.Components;
using Benner.BeTech.Tarefas.Components.ConclusaoTarefa;
using Benner.Tecnologia.Business;
using Benner.Tecnologia.Common.Services;
using Ninject;
using System;
using System.Linq;

namespace Benner.BeTech.Tarefas.Components
{
    internal sealed class ConclusaoTarefaComponent :
        BusinessComponent<ConclusaoTarefaComponent>,
        IConclusaoTarefaComponent,
        IInitializable
    {
        private DateTime _agora;

        [Inject] public ITransactionContextFactory TransactionContextFactory { get; set; }
        [Inject] public IBt_UsuarioTarefasDao BtUsuarioTarefasDao { get; set; }

        [Inject] public IProcessador[] Processadores { get; set; }
        [Inject] public IValidador[] Validadores { get; set; }


        [Inject]
        public void InjetarDependencia(IAdministrationService dependencia)
        {
            _agora = dependencia.ServerDateTime();
        }

        public void Cancelar(long btUsuarioTarefaHandle)
        {
            var entity = BtUsuarioTarefasDao.Get(btUsuarioTarefaHandle);

            using (var tc = TransactionContextFactory.Begin())
            {
                foreach (var validador in Validadores)
                {
                    validador.ValidarCancelamento(entity);
                }

                foreach (var processador in Processadores.OrderBy(p => p.CustomizationLevel))
                {
                    processador.Cancelar(entity);
                }

                BtUsuarioTarefasDao.Save(entity);

                tc.Complete();
            }
        }

        public void Concluir(ProcessarConclusaoRequest request)
        {
            var entity = BtUsuarioTarefasDao.Get(request.BtUsuarioTarefaHandle);

            using (var tc = TransactionContextFactory.Begin())
            {
                foreach (var validador in Validadores)
                {
                    validador.ValidarConclusao(entity, request);
                }

                foreach (var processador in Processadores.OrderBy(p => p.CustomizationLevel))
                {
                    processador.Concluir(entity, request);
                }

                BtUsuarioTarefasDao.Save(entity);

                tc.Complete();
            }
        }

        public void Initialize()
        {

        }
    }
}
